#include <math.h>
#include "mex.h"

/*
 *
 *  mss_sd.c:
 *  MEX-file for [a,b]=nlid_gsum(x)
 *  for m-by-1 double x generates n-by-1 doubles a and b, where
 *  a=[a1(:);x(1)+a2(:);sum(x(1:2))+a3(:);...;sum(x(1:m-1))+am(:)] 
 *  ai=repmat(1:x(i),x(i),1)
 *  b=[b1(:);x(1)+b2(:);sum(x(1:2))+b3(:);...;sum(x(1:m-1))+bm(:)]
 *  bi=repmat(1:x(i),x(i),1)
 */


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] ) {
  double *a,*b,*x;              /* input x, output y      */
  double z;
  mwSize m,n,N,q,qq,i,iqq,j,imx,k;              /* integer counters */
  
  if(nrhs!=1) 
      mexErrMsgTxt("exactly one input required");
  if(nlhs!=2) 
      mexErrMsgTxt("exactly two outputs required");
  if( !mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) ) 
      mexErrMsgTxt("input must be a noncomplex double");
  m = mxGetM(prhs[0]);
  N = mxGetN(prhs[0]);
  if((m==0) || (N!=1) )
      mexErrMsgTxt("input is not a column vector");
  x=mxGetPr(prhs[0]);
  qq=0;
  for(i=0;i<m;i++) {
      q = (mwSize)x[i];
      /* mexPrintf("\n x[%d]=%d\n", i+1,q); */
      qq+=(q*(q+1))/2;
  }
  plhs[0] = mxCreateDoubleMatrix(qq,1, mxREAL);
  plhs[1] = mxCreateDoubleMatrix(qq,1, mxREAL);
  a = mxGetPr(plhs[0]);
  b = mxGetPr(plhs[1]);
  q=0;
  qq=0;
  for(k=0;k<m;k++) {
    imx = (mwSize)x[k];
    for(i=1;i<=imx;i++) {
      iqq = i+qq;
      for(j=1;j<=i;j++) {
          a[q]=(double)iqq;
          b[q]=(double)(j+qq);
          q++;
      }
    }
    qq+=imx;
  }
}
  
  